# %%

from pathlib import Path
import pandas as pd
import numpy as np


KEEP_REDCAP_COLS = [
    "record_id",
    "locationid",
    "groupid",
    "start_time",
    "end_time",
    "latitude",
    "longitude",
    "ssi01",
    "ssi02",
    "ssi03",
    "ssi04",
    "paq01",
    "paq02",
    "paq03",
    "paq04",
    "paq05",
    "paq06",
    "paq07",
    "paq08",
    "sss01",
    "sss02",
    "sss03",
    "sss04",
    "sss05",
    "who01",
    "who02",
    "who03",
    "who04",
    "who05",
    "age00",
    "gen00",
    "edu00",
    "eth00",
    "occ00___1",
    "occ00___2",
    "occ00___3",
    "occ00___4",
    "occ00___5",
    "occ00___6",
    "eth00_other",
    "occ00_other",
    "misc03",
    "misc03_other",
    "misc01",
    "use00",
    "uni00",
]

REDCAP_PAQ_RENAMES = {
    "paq01": "pleasant",
    "paq02": "chaotic",
    "paq03": "vibrant",
    "paq04": "uneventful",
    "paq05": "calm",
    "paq06": "annoying",
    "paq07": "eventful",
    "paq08": "monotonous",
}

KEEP_ZHENZHEN_COLS = [
    "record_id",
    "sessionid",
    "locationid",
    "groupid",
    "start_time",
    "end_time",
    "latitude",
    "longitude",
    "ssi01",
    "ssi02",
    "ssi03",
    "ssi04",
    "paq01",
    "paq02",
    "paq03",
    "paq04",
    "paq05",
    "paq06",
    "paq07",
    "paq08",
    "sss01",
    "sss02",
    "sss03",
    "sss04",
    "sss05",
    "who01",
    "who02",
    "who03",
    "who04",
    "who05",
    "age00",
    "gen00",
    "edu00",
    "eth00",
    "occ00",
    "eth00_other",
    "occ00_other",
    "misc03",
    "misc03_other",
    "misc01",
    "use00",
    "uni00",
]

KEEP_SHENYANG_COLS = [
    "record_id",
    "LocationID",
    "SessionID",
    "GroupID",
    "QueVer",
    "start_time",
    "Unnamed: 8",
    "Traffic",
    "Human",
    "Natural",
    "Other",
    "pleasant",
    "chaotic",
    "vibrant",
    "uneventful",
    "calm",
    "annoying",
    "eventful",
    "monotonous",
    "overall",
    "appropriateness",
    "loudness",
    "sss04",
    "sss05",
    "who01",
    "who02",
    "who03",
    "who04",
    "who05",
    "Age",
    "Gender",
    "edu00",
    "occ00",
    "misc03",
    "misc01",
    "use00",
    "uni00",
]

EDU00_DICT = {
    1: "SomeHighSchool",
    2: "HighSchoolGraduate",
    3: "SomeCollege",
    4: "TradeTechVocation",
    5: "UniversityGraduate",
    6: "SomePostgraduate",
    7: "PostgraduateDegree",
}

OCC00_DICT = {
    1: "Employed",
    2: "Unemployed",
    3: "Retired",
    4: "Student",
    5: "Other",
    6: "RatherNotSay",
}

MISC03_DICT = {
    1: "Local",
    2: "Tourist",
    3: "Other",
}

USE00_DICT = {
    1: "Staying",
    2: "Arriving",
    3: "Leaving",
    4: "PassingThrough",
}

UNI00_DICT = {
    1: "Alone",
    2: "Couple",
    3: "Group",
}

ETH00_DICT = {
    1: "White",
    2: "MixedMultipleEthnicGroups",
    3: "AsianAsianBritish",
    4: "BlackAfricanCaribbeanBlackBritish",
    5: "MiddleEastern",
    6: "RatherNotSay",
    7: "Other",
}

FINAL_COLS = [
    "LocationID",
    "SessionID",
    "GroupID",
    "RecordID",
    "start_time",
    "end_time",
    "latitude",
    "longitude",
    "Language",
    "Survey_Version",
    "ssi01",
    "ssi02",
    "ssi03",
    "ssi04",
    "pleasant",
    "chaotic",
    "vibrant",
    "uneventful",
    "calm",
    "annoying",
    "eventful",
    "monotonous",
    "sss01",
    "sss02",
    "sss03",
    "sss04",
    "sss05",
    "who01",
    "who02",
    "who03",
    "who04",
    "who05",
    "WHO_Sum",
    "age00",
    "gen00",
    "edu00",
    "eth00",
    "occ00",
    "eth00_other",
    "occ00_other",
    "misc03",
    "misc03_other",
    "misc01",
    "use00",
    "uni00",
]


# %%


def multichoice_occ_recode(data):
    data["occ00___1"] = data["occ00___1"].replace({1: "Employed", 0: np.nan})
    data["occ00___2"] = data["occ00___2"].replace({1: "Unemployed", 0: np.nan})
    data["occ00___3"] = data["occ00___3"].replace({1: "Retired", 0: np.nan})
    data["occ00___4"] = data["occ00___4"].replace({1: "Student", 0: np.nan})
    data["occ00___5"] = data["occ00___5"].replace({1: "Other", 0: np.nan})
    data["occ00___6"] = data["occ00___6"].replace({1: "RatherNotSay", 0: np.nan})
    data["occ00"] = data[
        ["occ00___1", "occ00___2", "occ00___3", "occ00___4", "occ00___5", "occ00___6"]
    ].apply(lambda x: ", ".join(x.dropna()), axis=1)
    data.drop(
        columns=[
            "occ00___1",
            "occ00___2",
            "occ00___3",
            "occ00___4",
            "occ00___5",
            "occ00___6",
        ],
        inplace=True,
    )

    return data


def categorical_recode(data, col, dict):
    data[col] = data[col].replace(dict)
    return data


def load_and_clean_ssid_2021(file, lang, surv_version):
    assert lang in ["eng", "spa", "ita"]
    assert surv_version in ["ISO2018", "SSIDv1"]
    assert file.exists()

    data = pd.read_csv(file, header=0)

    # Drop the columns that are not needed
    data = data[KEEP_REDCAP_COLS]

    # Convert the start and end times to datetime
    if lang == "spa":
        data["start_time"] = pd.to_datetime(data["start_time"], format="%d/%m/%Y %H:%M")
        data["end_time"] = pd.to_datetime(data["end_time"], format="%d/%m/%Y %H:%M")

    else:
        data["start_time"] = pd.to_datetime(data["start_time"])
        data["end_time"] = pd.to_datetime(data["end_time"])

    # Rename PAQ columns
    data.rename(columns=REDCAP_PAQ_RENAMES, inplace=True)

    # Rename Index Columns
    data.rename(
        columns={
            "locationid": "SessionID",
            "groupid": "GroupID",
            "record_id": "RecordID",
        },
        inplace=True,
    )

    # Recode Gender data
    data["gen00"] = data["gen00"].replace(
        {1.0: "Male", 2.0: "Female", 3.0: "Non-conforming", 4.0: "RatherNotSay"}
    )

    # Recode Occupation data
    data = multichoice_occ_recode(data)
    data = categorical_recode(data, "edu00", EDU00_DICT)
    data = categorical_recode(data, "misc03", MISC03_DICT)
    data = categorical_recode(data, "use00", USE00_DICT)
    data = categorical_recode(data, "uni00", UNI00_DICT)
    data = categorical_recode(data, "eth00", ETH00_DICT)

    # Add Language
    data["Language"] = lang

    # Add survey version
    data["Survey_Version"] = lang + surv_version

    # Uppercase GroupID
    data["GroupID"] = data["GroupID"].str.upper()

    # Rename Groningen SessionIDs
    if lang == "eng" or lang == "nld":
        data["SessionID"] = data["SessionID"].replace(
            {
                "Groningen-Noorderplantsoen": "Noorderplantsoen1",
            }
        )
        # Replace Noorderplantsoen GroupIDs
        data.loc[data["GroupID"].str.contains("GRNPS"), "GroupID"] = data[
            "GroupID"
        ].str.replace("GRNPS", "NP")

    # Add LocationID
    data["LocationID"] = data["SessionID"].str.extract(r"(\D+)")

    return data


def load_and_clean_ssid_nld_2020(file, lang="nld", surv_version="SSIDv1"):
    assert file.exists()

    data = pd.read_csv(file, header=0)

    # Drop the columns that are not needed
    KEEP_COLS = KEEP_REDCAP_COLS.copy()
    KEEP_COLS.remove("groupid")
    KEEP_COLS = KEEP_COLS + ["GroupID", "SessionID"]
    data = data[KEEP_COLS]

    # Convert the start and end times to datetime
    data["start_time"] = pd.to_datetime(data["start_time"], format="%d/%m/%Y %H:%M")
    data["end_time"] = pd.to_datetime(data["end_time"], format="%d/%m/%Y %H:%M")

    # Rename PAQ columns
    data.rename(columns=REDCAP_PAQ_RENAMES, inplace=True)

    # Recode Gender data
    data["gen00"] = data["gen00"].replace(
        {1.0: "Male", 2.0: "Female", 3.0: "Non-conforming", 4.0: "RatherNotSay"}
    )
    # data.rename(columns={'gen00': 'Gender'}, inplace=True)

    # Recode Occupation data
    data = multichoice_occ_recode(data)
    data = categorical_recode(data, "edu00", EDU00_DICT)
    data = categorical_recode(data, "misc03", MISC03_DICT)
    data = categorical_recode(data, "use00", USE00_DICT)
    data = categorical_recode(data, "uni00", UNI00_DICT)
    data = categorical_recode(data, "eth00", ETH00_DICT)

    # Add Language
    data["Language"] = lang

    # Add survey version
    data["Survey_Version"] = lang + surv_version

    # Uppercase GroupID
    data["GroupID"] = data["GroupID"].str.upper()

    # Rename locationid
    data.rename(
        columns={"locationid": "LocationID", "record_id": "RecordID"}, inplace=True
    )

    # Add LocationID
    data["LocationID"] = data["SessionID"].str.extract(r"(\D+)")

    return data


def load_and_clean_shenyang_2022(file):
    assert file.exists()

    data = pd.read_excel(file, header=0)
    data = data.iloc[5:]

    # Drop the columns that are not needed
    data = data[KEEP_SHENYANG_COLS]

    # Rename Some Columns
    data.rename(
        columns={"Unnamed: 8": "end_time", "record_id": "RecordID"}, inplace=True
    )
    data.rename(
        columns={
            "Traffic": "ssi01",
            "Human": "ssi02",
            "Natural": "ssi03",
            "Other": "ssi04",
        },
        inplace=True,
    )
    data.rename(
        columns={"overall": "sss01", "appropriateness": "sss02", "loudness": "sss03"},
        inplace=True,
    )
    data.rename(columns={"Age": "age00", "Gender": "gen00"}, inplace=True)

    # Convert the start and end times to datetime
    data["start_time"] = pd.to_datetime(data["start_time"])
    data["end_time"] = pd.to_datetime(data["end_time"])

    # Add Language
    data["Language"] = "cmn"

    # Add survey version
    data["Survey_Version"] = data["QueVer"].replace(
        {
            1: "cmnSSIDv1",
            2: "cmnSSIDv2",
            3: "cmnSSIDv3",
        }
    )
    data.drop(columns=["QueVer"], inplace=True)

    # Uppercase GroupID
    data["GroupID"] = data["GroupID"].str.upper()

    # Remove known bad data
    data = data[data["RecordID"] != "WJ05"]
    data = data[data["RecordID"] != "WJ07"]

    # Recode occupation
    data = categorical_recode(data, "edu00", EDU00_DICT)
    data = categorical_recode(data, "misc03", MISC03_DICT)
    data = categorical_recode(data, "use00", USE00_DICT)
    data = categorical_recode(data, "uni00", UNI00_DICT)
    data = categorical_recode(data, "occ00", OCC00_DICT)

    return data


def load_and_clean_shenzhen_2022(file):
    assert file.exists()

    data = pd.read_excel(file, header=1)

    # Drop the columns that are not needed
    data = data[KEEP_ZHENZHEN_COLS]

    # Convert the start and end times to datetime
    data["start_time"] = pd.to_datetime(data["start_time"])
    data["end_time"] = pd.to_datetime(data["end_time"])

    # Rename Index Columns
    data.rename(
        columns={
            "record_id": "RecordID",
            "locationid": "LocationID",
            "sessionid": "SessionID",
            "groupid": "GroupID",
        },
        inplace=True,
    )
    data.rename(columns=REDCAP_PAQ_RENAMES, inplace=True)

    # Recode
    data["gen00"] = data["gen00"].replace(
        {
            1: "Male",
            2: "Female",
        }
    )

    # Add Language
    data["Language"] = "cmn"

    # Add survey version
    # TODO: Check survey version
    data["Survey_Version"] = "cmnSSIDv3"

    # Uppercase GroupID
    data["GroupID"] = data["GroupID"].str.upper()

    # Recode occupation
    data = categorical_recode(data, "edu00", EDU00_DICT)
    data = categorical_recode(data, "misc03", MISC03_DICT)
    data = categorical_recode(data, "use00", USE00_DICT)
    data = categorical_recode(data, "uni00", UNI00_DICT)
    data = categorical_recode(data, "occ00", OCC00_DICT)

    return data


def concat_ssid_data(lang_dfs: list[pd.DataFrame]) -> pd.DataFrame:
    df = pd.concat(lang_dfs)

    # Add WHO_Sum
    df["WHO_Sum"] = df[["who01", "who02", "who03", "who04", "who05"]].sum(axis=1) * 4

    # Reorder columns
    assert set(FINAL_COLS) == set(df.columns)
    df = df[FINAL_COLS]

    return df
