# %%

### Script for reformatting and combining the 2 channel data from ArtemiS psychoacoustic output.

import pandas as pd
from pathlib import Path
import numpy as np
from datetime import datetime


def load_and_clean_artemis_data(file):

    # Load all sheets and concatenate
    df = pd.concat(pd.read_excel(file, sheet_name=None, skiprows=2, header=None))
    df = df.reset_index(level=1, drop=True).rename_axis("filenames").reset_index()

    filenames = df.filenames.values
    df.drop(columns="filenames", inplace=True)

    # Re-name columns by combining first two ArtemiS rows
    cols = pd.read_excel(file, nrows=2, header=None).replace(np.nan, "missing")
    df.columns = [f"{cols[name][0]}_{cols[name][1]}" for name in cols.columns]

    # SessionID and GroupID columns

    df["SessionID"] = [file.split("_")[-1] for file in filenames]
    df["GroupID"] = df.iloc[:, 0].str.split(" ").str[0].astype("string")
    df["GroupID"] = df["GroupID"].str.upper()
    df["RecordingLength"] = df.iloc[:, 0].str.split(" ").str[3].astype("float")

    df.columns.values[1] = "channel"

    # Clean up columns

    df = df[df.columns.drop(list(df.filter(regex="missing")))]
    df = df[df.columns.drop(list(df.filter(regex="@")))]

    return df


def process_artemis_data(df):
    # Split channels

    df_left = df[df.channel == "Left"].copy()
    df_right = df[df.channel == "Right"].copy()

    df_left.index = df_left.GroupID
    df_left.drop(["channel", "GroupID"], axis=1, inplace=True)

    df_right.index = df_right.GroupID
    df_right.drop(["channel", "GroupID"], axis=1, inplace=True)

    # Calculate channel combinations and save as separate dataframes
    df_2ch = df_left[["SessionID", "RecordingLength"]]
    df_max = df_left[["SessionID", "RecordingLength"]]
    df_min = df_left[["SessionID", "RecordingLength"]]
    df_diff = df_left[["SessionID", "RecordingLength"]]
    df_avg = df_left[["SessionID", "RecordingLength"]]

    for col in df_left.columns:
        if col not in ["SessionID", "RecordingLength"]:
            vals = pd.DataFrame(
                {
                    f"{col}_Left": df_left[col].values,
                    f"{col}_Right": df_right[col].values,
                },
                index=df_2ch.index,
            )

            df_2ch = pd.merge(df_2ch, vals, left_index=True, right_index=True)

            max_vals = df_2ch[[f"{col}_Left", f"{col}_Right"]].max(axis=1)
            max_vals = pd.DataFrame(
                max_vals, columns=[f"{col}"], index=df_max.index
            )
            df_max = pd.merge(df_max, max_vals, left_index=True, right_index=True)

            min_vals = df_2ch[[f"{col}_Left", f"{col}_Right"]].min(axis=1)
            min_vals = pd.DataFrame(
                min_vals, columns=[f"{col}"], index=df_min.index
            )
            df_min = pd.merge(df_min, min_vals, left_index=True, right_index=True)

            diff_vals = np.absolute(df_2ch[f"{col}_Left"] - df_2ch[f"{col}_Right"])
            diff_vals = pd.DataFrame(
                diff_vals, columns=[f"{col}"], index=df_diff.index
            )
            df_diff = pd.merge(df_diff, diff_vals, left_index=True, right_index=True)

            avg_vals = (df_2ch[f"{col}_Left"] + df_2ch[f"{col}_Right"]) / 2
            avg_vals = pd.DataFrame(
                avg_vals, columns=[f"{col}"], index=df_avg.index
            )
            df_avg = pd.merge(df_avg, avg_vals, left_index=True, right_index=True)

    return df_2ch, df_max, df_min, df_diff, df_avg
