# %%

from ISDcleaning import *
from ISDpsycho import *
from pathlib import Path
import pandas as pd
from datetime import datetime


def compile_all_v10(source_dir, out_dir, interim_dir=None, psycho_metric="max"):
    now = datetime.now()
    now_str = now.strftime("%Y-%m-%d_%H%M")

    # Load data
    eng_file = source_dir.joinpath(
        "SSIDEnglish-OFFAllCompleted_DATA_2021-04-29_2219.csv"
    )
    spa_file = source_dir.joinpath(
        "SSIDSpanish-AllCompleteOFF_DATA_2021-04-30_1127.csv"
    )
    ita_file = source_dir.joinpath(
        "SSIDItalian-AllCompletedOFF_DATA_2021-04-30_1140.csv"
    )
    nld_file = source_dir.joinpath("CORRECTED_SSIDDutch_DATA_2020-03-13_1610 copy.csv")
    shny_file = source_dir.joinpath("All questionnaires_OFF_Shenyang_20Apr22.xlsx")
    szhn_file = source_dir.joinpath("OFF_ZhenzhenCombined_QUE.xlsx")

    print("Loading and cleaning English data...")
    eng_data = load_and_clean_ssid_2021(eng_file, "eng", "ISO2018")

    print("Loading and cleaning Spanish data...")
    spa_data = load_and_clean_ssid_2021(spa_file, "spa", "SSIDv1")

    print("Loading and cleaning Italian data...")
    ita_data = load_and_clean_ssid_2021(ita_file, "ita", "SSIDv1")

    print("Loading and cleaning Dutch data...")
    nld_data = load_and_clean_ssid_nld_2020(nld_file)

    print("Loading and cleaning Shenzhen data...")
    szhn_data = load_and_clean_shenzhen_2022(szhn_file)

    print("Loading and cleaning Shenyang data...")
    shny_data = load_and_clean_shenyang_2022(shny_file)

    # Concatenate survey data
    print("\nConcatenating survey data...")
    survey_df = concat_ssid_data(
        [eng_data, spa_data, ita_data, nld_data, szhn_data, shny_data]
    )

    # Load and clean ArtemiS data
    print("\nLoading and cleaning Euro ArtemiS data...")
    euro_file = source_dir.joinpath(
        "OFF_AllEurope_BIN_SingleValue2ch_2021-10-15.xlsx"
    )
    euro_data = load_and_clean_artemis_data(euro_file)
    euro_2ch, euro_max, euro_min, euro_diff, euro_avg = process_artemis_data(euro_data)

    print("\nLoading and cleaning Shenzhen ArtemiS data...")
    shnzn_file = source_dir.joinpath("ShenzhenAnalysis1_2022-04-19.xlsx")
    shnzn_psycho = load_and_clean_artemis_data(shnzn_file)
    shnzn_2ch, shnzn_max, shnzn_min, shnzn_diff, shnzn_avg = process_artemis_data(
        shnzn_psycho
    )

    print("\nLoading and cleaning Shenyang ArtemiS data...")
    shny_file = source_dir.joinpath("OFF_ShenyangAnalysis1_2024-02-16.xlsx")
    shny_psycho = load_and_clean_artemis_data(shny_file)
    shny_2ch, shny_max, shny_min, shny_diff, shny_avg = process_artemis_data(shny_psycho)

    print("\nConcatenating psychoacoustic data...")
    df_2ch = pd.concat([euro_2ch, shnzn_2ch, shny_2ch], verify_integrity=True, ignore_index=False)
    df_max = pd.concat([euro_max, shnzn_max, shny_max], verify_integrity=True, ignore_index=False)
    df_min = pd.concat([euro_min, shnzn_min, shny_min], verify_integrity=True, ignore_index=False)
    df_diff = pd.concat([euro_diff, shnzn_diff, shny_diff], verify_integrity=True, ignore_index=False)
    df_avg = pd.concat([euro_avg, shnzn_avg, shny_avg], verify_integrity=True, ignore_index=False)
    
    # Merge ArtemiS and survey data
    if psycho_metric == "max":
        psycho_df = df_max
    elif psycho_metric == "min":
        psycho_df = df_min
    elif psycho_metric == "diff":
        psycho_df = df_diff
    elif psycho_metric == "avg":
        psycho_df = df_avg
    else:
        raise ValueError("psycho_metric must be one of 'max', 'min', 'diff', or 'avg'")

    print("\nMerging ArtemiS and survey data...")
    merged_df = pd.merge(survey_df, psycho_df, on=["GroupID", "SessionID"], how="left")

    # Save data
    print(f"\nSaving main data file - {out_dir.joinpath(f'ISD v1.0 Data.csv')}")
    merged_df.to_csv(out_dir.joinpath(f"ISD v1.0 Data.csv"), index=False)

    if interim_dir:
        if not interim_dir.exists():
            interim_dir.mkdir()
        print(f"\nSaving interim files to {interim_dir}/...")
        eng_data.to_csv(interim_dir.joinpath(f"eng_data_{now_str}.csv"), index=False)
        spa_data.to_csv(interim_dir.joinpath(f"spa_data_{now_str}.csv"), index=False)
        ita_data.to_csv(interim_dir.joinpath(f"ita_data_{now_str}.csv"), index=False)
        nld_data.to_csv(interim_dir.joinpath(f"nld_data_{now_str}.csv"), index=False)
        szhn_data.to_csv(interim_dir.joinpath(f"szhn_data_{now_str}.csv"), index=False)
        shny_data.to_csv(interim_dir.joinpath(f"shny_data_{now_str}.csv"), index=False)
        df_2ch.to_csv(interim_dir.joinpath(f"df_2ch_{now_str}.csv"))
        df_max.to_csv(interim_dir.joinpath(f"df_max_{now_str}.csv"))
        df_min.to_csv(interim_dir.joinpath(f"df_min_{now_str}.csv"))
        df_diff.to_csv(interim_dir.joinpath(f"df_diff_{now_str}.csv"))
        df_avg.to_csv(interim_dir.joinpath(f"df_avg_{now_str}.csv"))

    return merged_df


# %%

if __name__ == "__main__":
    """Compile ISD data from source files and save to output directory

    Example usage:
    python -m ISDv1_main --source_dir ../Survey_Data/source_data/ --out_dir ../ --interim_dir ../Survey_data/Interim_2024-02-08_cleaned --psycho_metric max
    """
    import argparse

    parser = argparse.ArgumentParser(description="Compile ISD data")
    parser.add_argument(
        "--source_dir",
        type=str,
        help="Directory containing source data",
        default="Survey_Data/source_data",
    )
    parser.add_argument(
        "--out_dir",
        type=str,
        help="Directory to save output",
        default="Survey_Data/output",
    )
    parser.add_argument(
        "--interim_dir",
        type=str,
        help="Directory to save interim files",
        default="Survey_Data/output/Interim",
    )
    parser.add_argument(
        "--psycho_metric",
        type=str,
        help="Which ArtemiS metric to use (max, min, diff, avg)",
        default="max",
    )
    args = parser.parse_args()

    source_dir = Path(args.source_dir)
    out_dir = Path(args.out_dir)
    interim_dir = Path(args.interim_dir)
    psycho_metric = args.psycho_metric

    compile_all_v10(source_dir, out_dir, interim_dir, psycho_metric)
